﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;

namespace EditableGrid
{
    public class MyEditor : Panel
    {
        private TextBox _textBox;

        protected TextBox TextBox
        {
            get { return _textBox; }
            set { _textBox = value; }
        }

        public static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value",
                                                                    typeof(object), typeof(MyEditor), new UIPropertyMetadata(null,
                                                                    new PropertyChangedCallback(OnValueChanged), new CoerceValueCallback(OnCoerceValue)));

        public object Value
        {
            get { return (object)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }


        public static readonly RoutedEvent ValueChangedEvent = EventManager.RegisterRoutedEvent("ValueChanged", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(MyEditor));

        public event RoutedEventHandler ValueChanged
        {
            add
            {
                AddHandler(ValueChangedEvent, value);
            }
            remove
            {
                RemoveHandler(ValueChangedEvent, value);
            }
        }

        public MyEditor()
        {
            this.TextBox = new TextBox();
            this.TextBox.TextChanged += new TextChangedEventHandler(TextBox_TextChanged);
            this.Children.Add(this.TextBox);
        }

        void TextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            this.Value = this.TextBox.Text;
        }


        private static void OnValueChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            MyEditor lookupTextBox = o as MyEditor;
            if (lookupTextBox != null)
                lookupTextBox.OnValueChanged((String)e.OldValue, (String)e.NewValue);
        }

        private static object OnCoerceValue(DependencyObject o, Object value)
        {
            MyEditor lookupTextBox = o as MyEditor;
            if (lookupTextBox != null)
            {
                if (value is String || value == null)
                    return lookupTextBox.OnCoerceValue((String)value);
                return lookupTextBox.OnCoerceValue((String)value.ToString());
            }
            else
                return value;
        }

        protected virtual String OnCoerceValue(String value)
        {
            return value;
        }

        protected virtual void OnValueChanged(String oldValue, String newValue)
        {
            if (this.Value != null)
                this.TextBox.Text = this.Value.ToString();
            else
                this.TextBox.Text = "";
            //
            // fire text changed event
            this.RaiseEvent(new RoutedEventArgs(MyEditor.ValueChangedEvent, this));
        }

    }
}
